@@ -1,5 +1,7 @@
 from shutil import copyfile
 
+import pytest
+
 from message_ix_models.tools.advance import (
     DIMS,
     LOCATION,
@@ -10,15 +12,21 @@ from message_ix_models.tools.advance import (
 from message_ix_models.util import package_data_path
 
 
-def test_get_advance_data(test_context):
-    """Test :func:`.get_advance_data`."""
+@pytest.fixture(scope="module")
+def advance_test_data(session_context):
     # Copy test data from the package directory into the local data directory for
     # `test_context`. get_advance_data() only uses this file if :mod:`message_data` is
     # NOT installed.
-    target = test_context.get_local_path(*LOCATION)
+    target = session_context.get_local_path(*LOCATION)
     target.parent.mkdir(parents=True)
     copyfile(package_data_path("test", *LOCATION), target)
 
+
+pytestmark = pytest.mark.usefixtures("advance_test_data")
+
+
+def test_get_advance_data(session_context):
+    """Test :func:`.get_advance_data`."""
     # Returns a pd.Series with the expected index levels
     result = get_advance_data()
     assert DIMS == result.index.names
@@ -28,7 +36,7 @@ def test_get_advance_data(test_context):
     assert {"[length]": 1, "[mass]": 1, "[time]": -1} == result.units.dimensionality
 
 
-def test_fuzz_data():
+def test_fuzz_data(session_context):
     """Test that :func:`_fuzz_data` runs successfully.
 
     NB this only produces a file in the :mod:`pytest` temporary directory. To update
@@ -37,5 +45,5 @@ def test_fuzz_data():
     """
     # size argument should be a fraction (~= 0.1) of the size of the test specimen
     _fuzz_data(
-        size=10, include=[("Transport|Service demand|Road|Freight", "billion tkm/yr")]
+        size=50, include=[("Transport|Service demand|Road|Freight", "billion tkm/yr")]
     )
