@@ -1,19 +1,43 @@
 """Tests of :mod:`message_ix_models.util.node`."""
 import re
 
+import pandas as pd
 import pytest
 from genno import Quantity
 from message_ix import Scenario, make_df
 
 from message_ix_models.model.bare import create_res
 from message_ix_models.model.structure import get_codes
-from message_ix_models.util import adapt_R11_R14, broadcast, identify_nodes
+from message_ix_models.util import (
+    adapt_R11_R12,
+    adapt_R11_R14,
+    broadcast,
+    identify_nodes,
+)
+from message_ix_models.util.node import MappingAdapter
+
+
+def test_mapping_adapter():
+    """Generic test of MappingAdapter."""
+    a = MappingAdapter({"foo": [("a", "x"), ("a", "y"), ("b", "z")]})
+
+    columns = ["foo", "bar", "value"]
+
+    df = pd.DataFrame([["a", "m", 1], ["b", "n", 2]], columns=columns)
+
+    result = a(df)
+
+    assert all(columns + ["unit"] == result.columns)
+
+    with pytest.raises(TypeError):
+        a(1.2)
+
 
 PAR = "technical_lifetime"
-VALUE = 0.1
+VALUE = [0.1, 0.2]
 
 
-@pytest.fixture(scope="module")
+@pytest.fixture(scope="function")
 def input():
     """Fixture: test data for :func:`.adapt_R11_R14`."""
     R11_all = get_codes("node/R11")
@@ -23,34 +47,45 @@ def input():
     ).pipe(broadcast, node_loc=R11_reg)
 
     # Set a specific value for the regions to be broadcast
-    df["value"] = df["value"].where(df["node_loc"] != "R11_FSU", VALUE)
+    df["value"] = df["value"].where(df["node_loc"] != "R11_CPA", VALUE[0])
+    df["value"] = df["value"].where(df["node_loc"] != "R11_FSU", VALUE[1])
 
     return {PAR: df}
 
 
-def test_adapt_R11_R14_0(input):
+@pytest.mark.parametrize(
+    "func,N,expected,target_nodes",
+    [
+        (adapt_R11_R12, 12, VALUE[0], ("R12_CHN", "R12_RCPA")),
+        (adapt_R11_R14, 14, VALUE[1], ("R14_CAS", "R14_RUS", "R14_SCS", "R14_UBM")),
+    ],
+)
+def test_adapt_df(input, func, N, expected, target_nodes):
     """:func:`.adapt_R11_R14` handles :class:`pandas.DataFrame`."""
     # Function runs
-    output = adapt_R11_R14(input)
+    output = func(input)
 
     # Output is a dict containing 1 entry
     df_out = output.pop(PAR)
     assert 0 == len(output)
 
     # Output covers all R14 regions
-    R14_all = get_codes("node/R14")
-    R14_reg = R14_all[R14_all.index("World")].child
-    assert set(R14_reg) == set(df_out["node_loc"])
+    all_nodes = get_codes(f"node/R{N}")
+    regions = all_nodes[all_nodes.index("World")].child
+    assert set(regions) == set(df_out["node_loc"])
 
     # Output has expected length
-    assert 14 * 2 == len(df_out)
+    assert N * 2 == len(df_out)
 
-    # Output values for new regions match input value for R11_FSU
-    target_nodes = ("R14_CAS", "R14_RUS", "R14_SCS", "R14_UBM")
-    assert (VALUE == df_out[df_out["node_loc"].isin(target_nodes)]["value"]).all()
+    # Output values for new regions match input value for the base region
+    assert (expected == df_out[df_out["node_loc"].isin(target_nodes)]["value"]).all()
 
 
-def test_adapt_R11_R14_1(input):
+@pytest.mark.parametrize(
+    "func,expected,node_loc",
+    [(adapt_R11_R12, VALUE[0], "R12_CHN"), (adapt_R11_R14, VALUE[1], "R14_CAS")],
+)
+def test_adapt_qty(input, func, expected, node_loc):
     """:func:`.adapt_R11_R14` handles :class:`genno.Quantity`."""
     # Convert to genno.Quantity
     df = input[PAR]
@@ -58,7 +93,7 @@ def test_adapt_R11_R14_1(input):
     input[PAR].attrs["_unit"] = df["unit"].unique()[0]
 
     # Function runs
-    output = adapt_R11_R14(input)
+    output = func(input)
 
     # Output is a dict containing 1 entry
     qty_out = output.pop(PAR)
@@ -67,9 +102,9 @@ def test_adapt_R11_R14_1(input):
     assert isinstance(qty_out, Quantity)
     assert "year" == qty_out.attrs["_unit"]
 
-    # Output values for new regions match input value for R11_FSU
+    # Output values for new regions match input value for the base region
     assert (
-        VALUE == qty_out.sel(node_loc="R14_CAS", technology="coal_ppl", year=2022)
+        expected == qty_out.sel(node_loc=node_loc, technology="coal_ppl", year=2022)
     ).all()
 
 
